""" SAM sdkit module

This scripts uses the sam sdk toolkit to simulate the performance of
a PV power plant. It uses the `sscapi.py` module that loads the
macOS SDK.

This script requires `pandas` and `numpy` to be insalled within the python
environment you are running this script

TODO:


See https://sam.nrel.gov/sites/default/files/content/virtual_conf_july_2013/07-sam-virtual-conference-2013-woodcock.pdf

"""

import sys
import log
import numpy as np
import pandas as pd
from pathlib import Path

# Package scripts
import sscapi
from utils import timeit
from utils import _create_data_folder, get_solar_data
from utils import check_nsrdb
from nsrdb import nsrdb
from nsrdb import get_nsrdb_data

# Package level variables
from context import *

ROOT =  Path(__file__).parents[1]

# Module level variables
DATA_DIR = ROOT / 'data'
DATA_DIR.mkdir(exist_ok=True, parents=True)
logger = log.custom_logger(__name__)

def default_sam_params():
    return ({
            'losses': 14.0757,
            'dc_ac_ratio': 1.1,
            'inv_eff': 96.,
            'system_capacity': 100,
            'configuration': 0, #  0 For fixed tilt, 2 for 1-axis and 4 for 2-axis
            'verbose': False,
            'gcr': 0.4,
            'tilt': 25,
            'azimuth':100}
            )


def sam(lat, lng, filename, force_download, year, verbose):
    site_info = {'lat': lat,
                 'lng': lng,
                 'force_download': force_download,
                 'year': str(year),
                 'filename': filename,
                 'verbose': verbose
                }
    if verbose:
        pp.pprint(site_info)

    solar_data = get_nsrdb_data(**site_info)
    simulation_params = {
                            'losses': 4.3,
                            'dc_ac_ratio': 1.2,
                            'inv_eff': 96.,
                            'system_capacity': 100,
                            'configuration': 2, #  0 For fixed tilt, 2 for 1-axis and 4 for 2-axis
                            'verbose': False}
    meta_data = get_nsrdb_data(meta=True, **site_info)
    simulation_params['elevation'] = meta_data['Elevation'].values
    simulation_params['timezone'] = meta_data['Time Zone'].values
    simulation_params['tilt'] = site_info['lat']
    z = {**simulation_params, **site_info}
    sam, _ = sam_simulation(solar_data, **z)
    pass

def sam_simulation(data, verbose=False, **kwargs):
    """ SAM solar PV simulation

    Perform a PVWATTS5 simulation using some input information about the
    solar plant.

    Parameters
    ----------
        data (pd.DataFrame): Solar radiation dataframe
        kwargs (dictionary): Dictionary containing simulation parameters

    Returns
    ----------
        CP (float): Capacity factor
        Generation (float): Generation over the year of simulation
        meto_data (pd.DataFrame): Dataframe with hourly generation
    """

    params = {'lat': kwargs['lat'],
              'lng': kwargs['lng'],
              'timezone': kwargs['timezone'],
              'elevation': kwargs['elevation'],
              'system_capacity': kwargs['system_capacity'],
              'dc_ac_ratio': kwargs['dc_ac_ratio'],
              'inv_eff': kwargs['inv_eff'],
              'losses': kwargs['losses'],
              'configuration': kwargs['configuration'],
              'tilt': kwargs['tilt'],
              'gcr': kwargs['gcr'],
              'azimuth': kwargs['azimuth'],
              'interval': kwargs['interval'],
              }

    if verbose:
        print ({key: value for key, value in params.items()})

    # Start sscapi module
    ssc = sscapi.PySSC()
    wfd = ssc.data_create()
    ssc.data_set_number(wfd, 'lat', params['lat'])
    ssc.data_set_number(wfd, 'lon', params['lng'])
    ssc.data_set_number(wfd, 'tz', params['timezone'])
    ssc.data_set_number(wfd, 'elev', params['elevation'])
    ssc.data_set_array(wfd, 'year', data.index.year)
    ssc.data_set_array(wfd, 'month', data.index.month)
    ssc.data_set_array(wfd, 'day', data.index.day)
    ssc.data_set_array(wfd, 'hour', data.index.hour)
    ssc.data_set_array(wfd, 'minute', data.index.minute)
    ssc.data_set_array(wfd, 'dn', data['DNI'])
    ssc.data_set_array(wfd, 'df', data['DHI'])
    ssc.data_set_array(wfd, 'wspd', data['Wind Speed'])
    ssc.data_set_array(wfd, 'tdry', data['Temperature'])

    # Create SAM compliant object  
    dat = ssc.data_create()
    ssc.data_set_table(dat, 'solar_resource_data', wfd)
    ssc.data_free(wfd)

    # Set system capacity in MW
    ssc.data_set_number(dat, 'system_capacity', params['system_capacity'])

    # Set DC/AC ratio (or power ratio).     
    ssc.data_set_number(dat, 'dc_ac_ratio', params['dc_ac_ratio'])

    # Set tilt of system in degrees
    ssc.data_set_number(dat, 'tilt', params['tilt'])

    # Set azimuth angle (in degrees) from north (0 degrees)
    ssc.data_set_number(dat, 'azimuth',params['azimuth'])

    # Set the inverter efficency
    ssc.data_set_number(dat, 'inv_eff',  params['inv_eff'])

    # Set the system losses, in percent
    ssc.data_set_number(dat, 'losses', params['losses'])

    # Set ground coverage ratio
    ssc.data_set_number(dat, 'gcr', params['gcr'])

    # Set constant loss adjustment
    ssc.data_set_number(dat, 'adjust:constant', 0)
    # system_capacity = params['sys_capacity']
    # value = params['configuration']
    ssc.data_set_number(dat, 'array_type', params['configuration'])

    mod = ssc.module_create('pvwattsv5')
    ssc.module_exec(mod, dat)

    # System AC generation
    data['ac_generation'] = np.array(ssc.data_get_array(dat, 'ac'))
    data['dc_generation'] = np.array(ssc.data_get_array(dat, 'dc'))
    data['generation'] = np.array(ssc.data_get_array(dat, 'gen'))

    # Plane of array (POA) irradiance in kWh
    data['POA'] = np.array(ssc.data_get_array(dat, 'poa'))

    # Module temperature in ºC
    data['TCell'] = np.array(ssc.data_get_array(dat, 'tcell'))

    capacity_factor = ssc.data_get_number(dat, 'capacity_factor')
    total_energy = ssc.data_get_number(dat, 'annual_energy')

    # free the memory
    ssc.data_free(dat)
    ssc.module_free(mod)

    # TODO: Check if this functionallity is what I want to implement as 
    # Standard

    # if isinstance(params['configuration'], dict):
        # d = {}
        # for key, val in value.iteritems():
            # ssc.data_set_number(dat, 'array_type', val)
            # # execute and put generation results back into dataframe
            # mod = ssc.module_create('pvwattsv5')
            # ssc.module_exec(mod, dat)
            # data['generation'] = np.array(ssc.data_get_array(dat, 'gen'))
            # CP = data['generation'].sum() / (525600/int('60') * system_capacity)
            # generation = data['generation'].sum()
            # d[key] = CP
            # d['gen_'+key] = generation
        # ssc.data_free(dat)
        # ssc.module_free(mod)
        # return (d)
    # else:
        # ssc.data_set_number(dat, 'array_type', value)
        # # execute and put generation results back into dataframe
        # mod = ssc.module_create('pvwattsv5')
        # ssc.module_exec(mod, dat)
        # data['generation'] = np.array(ssc.data_get_array(dat, 'gen'))
        # data['cf'] = data['generation'] / system_capacity
        # CP = data['generation'].sum() / (525600/int('60') * system_capacity)
        # generation = data['generation'].sum()
        # ssc.data_free(dat)
        # ssc.module_free(mod)
        # data.to_csv(output_path / 'test.csv')
        # return (data, (CP, generation))

    return (data, (capacity_factor, total_energy))


if __name__ == "__main__":
    site_info = {'lat': 33.21,
                 'lng': -97.12,
                 'force_download': False,
                 'year': str(2012),
                 'verbose': True,
                 'interval': 60
                }
    df = nsrdb(**site_info)
    meta_data = get_nsrdb_data(meta=True, **site_info)
    simulation_params = default_sam_params()
    simulation_params['elevation'] = meta_data['Elevation'].values
    simulation_params['timezone'] = meta_data['Time Zone'].values
    simulation_params['tilt'] = site_info['lat']

    # Include lat and lng to simulation params
    simulation_params.update(site_info)
    results, _ = sam_simulation(df, **simulation_params)
    print (results[['ac_generation', 'dc_generation', 'TCell']].head(24))



#Code to import packages
import sys
import os
from dotenv import load_dotenv
from pathlib import Path  # python3 only
env_path = Path('..') / '.env'
load_dotenv(dotenv_path=env_path)

ROOT = os.path.dirname(os.path.realpath(__file__))
sys.path.insert(0, ROOT)




 # -*- coding: utf-8 -*-
"""
Solar database
Example:
    Example of usage goes here

Attributes:
    module_level_variabless: None for the moment
Todo:
    * Implement multiple inputs and outputs for LCOE.
    * Clean code
"""
import sys
import numpy as np
import pandas as pd
import click
from pathlib import Path

import solar
import log
import sscapi
from utils import timeit
from utils import _create_data_folder, get_solar_data
from utils import check_nsrdb
from nsrdb import get_nsrdb_data
from context import *

ROOT =  Path(__file__).parents[1]

# Module level variables
DATA_DIR = ROOT / 'data'
DATA_DIR.mkdir(exist_ok=True, parents=True)
logger = log.custom_logger(__name__)


@click.command()
@click.option('--lat', default=19.5, prompt='Latitude',
                help='Latitude of the place')
@click.option('--lng', default=-99.5, prompt='Longitude',
                help='Longitude of the place')
@click.option('--year', default=2014, prompt='Year',
                help='Year of data up to 2016')
@click.option('--filename', default=False, prompt='Filename',
                help='Filename of the data', required=False)
@click.option('--force_download', default=False, is_flag=True,
                help='Force download from the API')
@click.option('--verbose', is_flag=True)
@click.pass_context
def sam(ctx, lat, lng, filename, force_download, year, verbose):
    click.secho('\nRunning SAM simulation...\n', fg='blue')
    site_info = {'lat': lat,
                 'lng': lng,
                 'force_download': force_download,
                 'year': str(year),
                 'filename': filename,
                 'verbose': verbose
                }
    if verbose:
        click.secho('Input data', fg='yellow')
        pp.pprint(site_info)

    solar_data = get_nsrdb_data(**site_info)
    simulation_params = {
                            'losses': 4.3,
                            'dc_ac_ratio': 1.2,
                            'inv_eff': 96.,
                            'system_capacity': 100,
                            'configuration': 2, #  0 For fixed tilt, 2 for 1-axis and 4 for 2-axis
                            'verbose': False}
    meta_data = get_nsrdb_data(meta=True, **site_info)
    simulation_params['elevation'] = meta_data['Elevation'].values
    simulation_params['timezone'] = meta_data['Time Zone'].values
    simulation_params['tilt'] = site_info['lat']
    z = {**simulation_params, **site_info}
    sam, _ = sam_simulation(solar_data, **z)
    pass

def sam_simulation(meteo_data, verbose=False, **kwargs):
    """ SAM performance simulation

    Perform a PVWATTS simulation using the SAM python implementation.
    Args:
        meteo_data (pd.DataFrame): Solar radiation dataframe
        kwargs (dictionary): Dictionary containing simulation parameters

    Returns:
        CP (float): Capacity factor
        Generation (float): Generation over the year of simulation
        meto_data (pd.DataFrame): Dataframe with hourly generation
    """
    params = {'lat': kwargs['lat'],
              'lon': kwargs['lng'],
              'timezone': kwargs['timezone'],
              'elevation': kwargs['elevation'],
              'sys_capacity': kwargs['system_capacity'],
              'dc_ac_ratio': kwargs['dc_ac_ratio'],
              'inv_eff': kwargs['inv_eff'],
              'losses': kwargs['losses'],
              'configuration': kwargs['configuration'],
              'tilt': kwargs['tilt'],
              }

    if verbose:
        print ({key: type(value) for key, value in params.items()})

    ssc = sscapi.PySSC()
    wfd = ssc.data_create()
    ssc.data_set_number(wfd, 'lat', params['lat'])
    ssc.data_set_number(wfd, 'lon', params['lon'])
    ssc.data_set_number(wfd, 'tz', params['timezone'])
    ssc.data_set_number(wfd, 'elev', params['elevation'])
    ssc.data_set_array(wfd, 'year', meteo_data.index.year)
    ssc.data_set_array(wfd, 'month', meteo_data.index.month)
    ssc.data_set_array(wfd, 'day', meteo_data.index.day)
    ssc.data_set_array(wfd, 'hour', meteo_data.index.hour)
    ssc.data_set_array(wfd, 'minute', meteo_data.index.minute)
    ssc.data_set_array(wfd, 'dn', meteo_data['DNI'])
    ssc.data_set_array(wfd, 'df', meteo_data['DHI'])
    ssc.data_set_array(wfd, 'wspd', meteo_data['Wind Speed'])
    ssc.data_set_array(wfd, 'tdry', meteo_data['Temperature'])

    # Create SAM compliant object  
    dat = ssc.data_create()
    ssc.data_set_table(dat, 'solar_resource_data', wfd)
    ssc.data_free(wfd)

    # Set system capacity in MW
    ssc.data_set_number(dat, 'system_capacity', params['sys_capacity'])

    # Set DC/AC ratio (or power ratio). See https://sam.nrel.gov/sites/default/files/content/virtual_conf_july_2013/07-sam-virtual-conference-2013-woodcock.pdf
    ssc.data_set_number(dat, 'dc_ac_ratio', params['dc_ac_ratio'])

    # Set tilt of system in degrees
    ssc.data_set_number(dat, 'tilt', params['tilt'])

    # Set azimuth angle (in degrees) from north (0 degrees)
    ssc.data_set_number(dat, 'azimuth', 180)

    # Set the inverter efficency
    ssc.data_set_number(dat, 'inv_eff',  params['inv_eff'])

    # Set the system losses, in percent
    ssc.data_set_number(dat, 'losses', params['losses'])

    # Set ground coverage ratio
    ssc.data_set_number(dat, 'gcr', 0.4)

    # Set constant loss adjustment
    ssc.data_set_number(dat, 'adjust:constant', 0)
    system_capacity = params['sys_capacity']
    value = params['configuration']
    if isinstance(params['configuration'], dict):
        d = {}
        for key, val in value.iteritems():
            ssc.data_set_number(dat, 'array_type', val)
            # execute and put generation results back into dataframe
            mod = ssc.module_create('pvwattsv5')
            ssc.module_exec(mod, dat)
            meteo_data['generation'] = np.array(ssc.data_get_array(dat, 'gen'))
            CP = meteo_data['generation'].sum() / (525600/int('60') * system_capacity)
            generation = meteo_data['generation'].sum()
            d[key] = CP
            d['gen_'+key] = generation
        ssc.data_free(dat)
        ssc.module_free(mod)
        return (d)
    else:
        ssc.data_set_number(dat, 'array_type', value)
        # execute and put generation results back into dataframe
        mod = ssc.module_create('pvwattsv5')
        ssc.module_exec(mod, dat)
        meteo_data['generation'] = np.array(ssc.data_get_array(dat, 'gen'))
        meteo_data['cf'] = meteo_data['generation'] / system_capacity
        CP = meteo_data['generation'].sum() / (525600/int('60') * system_capacity)
        generation = meteo_data['generation'].sum()
        click.secho('Output data from SAM.', fg='green')
        click.secho(f'Capacity Factor: {CP}', fg='blue')
        click.secho(f'Annual Generation: {generation}', fg='blue')
        ssc.data_free(dat)
        ssc.module_free(mod)
        meteo_data.to_csv(output_path / 'test.csv')
        return (meteo_data, (CP, generation))
    return (True)


if __name__ == "__main__":
    site_info = {'lat': 19.3,
                 'lng': -99.3,
                 'force_download': False,
                 'year': '2014'
                }

    solar_data = get_nsrdb_data(**site_info)
    simulation_params = {
                            'losses': 4.3,
                            'dc_ac_ratio': 1.2,
                            'inv_eff': 96.,
                            'system_capacity': 100,
                            'configuration': 2, #  0 For fixed tilt, 2 for 1-axis and 4 for 2-axis
                            'verbose': False}
    meta_data = get_nsrdb_data(meta=True, **site_info)
    simulation_params['elevation'] = meta_data['Elevation'].values
    simulation_params['timezone'] = meta_data['Time Zone'].values
    simulation_params['tilt'] = site_info['lat']
    z = {**simulation_params, **site_info}
    sam, _ = sam_simulation(solar_data, **z)
    pass



"""

TODO:
    * Implement multiple inputs and outputs for LCOE.
    * Modify Tracking method selection
"""

from __future__ import division, print_function
"""
Solar database
Example:
    Example of usage goes here

Attributes:
    module_level_variabless: None for the moment

"""
import pandas as pd
import sys

#if sys.version_info >= (3, 0):
#    sys.stdout.write("Sorry, requires Python 2.x, not Python 3.x\n")
#    sys.exit(1)


def lcoe(AEP, Tracking=None, *args, **kwargs):
    """LCOE cost estimation for solar PV plants
    Args:
        AEP: Annual energy production (pd.Series or Float)
        fixed: Fixed tilt (Boolean)
        Tracking: Tracking (Boolean)
        CapEx: Capital Expenditures [$/MW]
        OpEx: Operational expenditures [$/MW-y]
        d: Discount rate (%)
        n: Years of operation
        T: Tax rate [%]
        Pvdep: Depresation present value [%]

    Return:
        LCOE value in $/MW.
    """
    AEP = AEP/1000
    if Tracking == 0:
        CapEx = 1500000
        OpEx = 10000
    elif Tracking == 1:
        CapEx = 1620000
        OpEx = 10440
    elif Tracking == 2:
        CapEx = 2400000
        OpEx = 12000
    else:
        print ('No tracking informatio given')
    d = 0.06 
    n = 25.
    T = 0.3
    Pvdep = 0.8
    FCR = ((d*(1 + d)**n) / (d*(1 + d)**n) - 1) *((1 - (T*Pvdep))/(1 - T))
    LCOE = ((CapEx * FCR) + OpEx ) / AEP
    return (LCOE)

if __name__ == "__main__":
    print (lcoe(287116.310894))



# #####################################################################
#
#   System Simulation Core (SSC) Python Wrapper
#   Author: Aron Dobos @ NREL and Steven Janzou @ NREL
#
# #####################################################################


import string, sys, struct, os, numpy
from ctypes import *
from pathlib import Path

c_number = c_float # must be c_double or c_float depending on how defined in sscapi.h
class PySSC:

	def __init__(self):

		if sys.platform == 'win32' or sys.platform == 'cygwin':
			if 8*struct.calcsize("P") == 64:
				self.pdll = CDLL(str(Path(__file__).parents[0] /
                         "ssc.dll"))
			else:
				self.pdll = CDLL(str(Path(__file__).parents[0] /
                         "ssc.dll"))
		elif sys.platform == 'darwin':
			self.pdll = CDLL(Path(__file__).parents[0] / "ssc.dylib")
		elif sys.platform == 'linux2':
			self.pdll = CDLL('./ssc.so')   # instead of relative path, require user to have on LD_LIBRARY_PATH
		else:
			print ("Platform not supported ", sys.platform)


	INVALID=0
	STRING=1
	NUMBER=2
	ARRAY=3
	MATRIX=4

	INPUT=1
	OUTPUT=2
	INOUT=3

	def version(self):
		self.pdll.ssc_version.restype = c_int
		return self.pdll.ssc_version()

	def data_create(self):
		self.pdll.ssc_data_create.restype = c_void_p
		return self.pdll.ssc_data_create()

	def data_free(self, p_data):
		self.pdll.ssc_data_free( c_void_p(p_data) )

	def data_clear(self, p_data):
		self.pdll.ssc_data_clear( c_void_p(p_data) )

	def data_unassign(self, p_data, name):
		self.pdll.ssc_data_unassign( c_void_p(p_data), c_char_p(name.encode('utf-8')) )

	def data_query(self, p_data, name):
		self.pdll.ssc_data_query.restype = c_int
		return self.pdll.ssc_data_query( c_void_p(p_data), c_char_p(name.encode('utf-8')) )

	def data_first(self, p_data):
		self.pdll.ssc_data_first.restype = c_char_p
		return self.pdll.ssc_data_first( c_void_p(p_data) )

	def data_next(self, p_data):
		self.pdll.ssc_data_next.restype = c_char_p
		return self.pdll.ssc_data_next( c_void_p(p_data) )

	def data_set_string(self, p_data, name, value):
		self.pdll.ssc_data_set_string( c_void_p(p_data), c_char_p(name.encode('utf-8')), c_char_p(value.encode('utf-8')) )

	def data_set_number(self, p_data, name, value):
		self.pdll.ssc_data_set_number( c_void_p(p_data), c_char_p(name.encode('utf-8')), c_number(value) )

	def data_set_array(self,p_data,name,parr):
		count = len(parr)
		arr = (c_number*count)()
		arr[:] = parr # set all at once instead of looping
			
		return self.pdll.ssc_data_set_array( c_void_p(p_data), c_char_p(name.encode('utf-8')),pointer(arr), c_int(count))

	def data_set_matrix(self,p_data,name,mat):
		nrows = len(mat)
		ncols = len(mat[0])
		size = nrows*ncols
		arr = (c_number*size)()
		idx=0
		for r in range(nrows):
			for c in range(ncols):
				arr[idx] = c_number(mat[r][c])
				idx=idx+1
		return self.pdll.ssc_data_set_matrix( c_void_p(p_data), c_char_p(name.encode('utf-8')), pointer(arr), c_int(nrows), c_int(ncols))

	def data_set_table(self,p_data,name,tab):
		return self.pdll.ssc_data_set_table( c_void_p(p_data), c_char_p(name.encode('utf-8')), c_void_p(tab) );

	def data_get_string(self, p_data, name):
		self.pdll.ssc_data_get_string.restype = c_char_p
		return self.pdll.ssc_data_get_string( c_void_p(p_data), c_char_p(name.encode('utf-8')) )

	def data_get_number(self, p_data, name):
		val = c_number(0)
		self.pdll.ssc_data_get_number( c_void_p(p_data), c_char_p(name.encode('utf-8')), byref(val) )
		return val.value

	def data_get_array(self,p_data,name):
		count = c_int()
		self.pdll.ssc_data_get_array.restype = POINTER(c_number)
		parr = self.pdll.ssc_data_get_array( c_void_p(p_data), c_char_p(name.encode('utf-8')), byref(count))
		arr = parr[0:count.value] # extract all at once			
		return arr

	def data_get_matrix(self,p_data,name):
		nrows = c_int()
		ncols = c_int()
		self.pdll.ssc_data_get_matrix.restype = POINTER(c_number)
		parr = self.pdll.ssc_data_get_matrix( c_void_p(p_data), c_char_p(name.encode('utf-8')), byref(nrows), byref(ncols) )
		idx = 0
		mat = []
		for r in range(nrows.value):
			row = []
			for c in range(ncols.value):
				row.append( float(parr[idx]) )
				idx = idx + 1
			mat.append(row)
		return mat
		
	# don't call data_free() on the result, it's an internal
	# pointer inside SSC
	def data_get_table(self,p_data,name): 
		return self.pdll.ssc_data_get_table( c_void_p(p_data), name );

	def module_entry(self,index):
		self.pdll.ssc_module_entry.restype = c_void_p
		return self.pdll.ssc_module_entry( c_int(index) )

	def entry_name(self,p_entry):
		self.pdll.ssc_entry_name.restype = c_char_p
		return self.pdll.ssc_entry_name( c_void_p(p_entry) )

	def entry_description(self,p_entry):
		self.pdll.ssc_entry_description.restype = c_char_p
		return self.pdll.ssc_entry_description( c_void_p(p_entry) )

	def entry_version(self,p_entry):
		self.pdll.ssc_entry_version.restype = c_int
		return self.pdll.ssc_entry_version( c_void_p(p_entry) )

	def module_create(self,name):
		self.pdll.ssc_module_create.restype = c_void_p
		return self.pdll.ssc_module_create( c_char_p(name.encode('utf-8')) )

	def module_free(self,p_mod):
		self.pdll.ssc_module_free( c_void_p(p_mod) )

	def module_var_info(self,p_mod,index):
		self.pdll.ssc_module_var_info.restype = c_void_p
		return self.pdll.ssc_module_var_info( c_void_p(p_mod), c_int(index) )

	def info_var_type( self, p_inf ):
		return self.pdll.ssc_info_var_type( c_void_p(p_inf) )

	def info_data_type( self, p_inf ):
		return self.pdll.ssc_info_data_type( c_void_p(p_inf) )

	def info_name( self, p_inf ):
		self.pdll.ssc_info_name.restype = c_char_p
		return self.pdll.ssc_info_name( c_void_p(p_inf) )

	def info_label( self, p_inf ):
		self.pdll.ssc_info_label.restype = c_char_p
		return self.pdll.ssc_info_label( c_void_p(p_inf) )

	def info_units( self, p_inf ):
		self.pdll.ssc_info_units.restype = c_char_p
		return self.pdll.ssc_info_units( c_void_p(p_inf) )

	def info_meta( self, p_inf ):
		self.pdll.ssc_info_meta.restype = c_char_p
		return self.pdll.ssc_info_meta( c_void_p(p_inf) )

	def info_group( self, p_inf ):
		self.pdll.ssc_info_group.restype = c_char_p
		return self.pdll.ssc_info_group( c_void_p(p_inf) )

	def info_uihint( self, p_inf ):
		self.pdll.ssc_info_uihint.restype = c_char_p
		return self.pdll.ssc_info_uihint( c_void_p(p_inf) )

	def module_exec( self, p_mod, p_data ):
		self.pdll.ssc_module_exec.restype = c_int
		return self.pdll.ssc_module_exec( c_void_p(p_mod), c_void_p(p_data) )
		ssc_module_exec_simple_nothread
		
	def module_exec_simple_no_thread( self, modname, data ):
		self.pdll.ssc_module_exec_simple_nothread.restype = c_char_p;
		return self.pdll.ssc_module_exec_simple_nothread( c_char_p(modname.encode('utf-8')), c_void_p(data) );

	def module_log( self, p_mod, index ):
		log_type = c_int()
		time = c_float()
		self.pdll.ssc_module_log.restype = c_char_p
		return self.pdll.ssc_module_log( c_void_p(p_mod), c_int(index), byref(log_type), byref(time) )


	def module_exec_set_print( self, prn ):
		return self.pdll.ssc_module_exec_set_print( c_int(prn) );

if __name__ == "__main__":

	def setup_wind(ssc, data):
		ssc.data_set_number( data, 'wind_resource_shear', 0.14 );
		ssc.data_set_number( data, 'wind_resource_turbulence_coeff', 0.1 );
		ssc.data_set_number( data, 'system_capacity', 2.4 );
		ssc.data_set_number( data, 'wind_resource_model_choice', 0 );
		ssc.data_set_number( data, 'wind_characteristics_weibullK', 2.1 );
		ssc.data_set_number( data, 'wind_characteristics_class', 7.6 );
		ssc.data_set_number( data, 'wind_turbine_rotor_diameter', 3.7 );
		ssc.data_set_array( data, 'wind_turbine_powercurve_windspeeds', [ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40 ] );
		ssc.data_set_array( data, 'wind_turbine_powercurve_powerout', [ 0, 0, 0, 0, 0.08, 0.2, 0.35, 0.6, 1, 1.6, 2, 2.25, 2.35, 2.4, 2.4, 2.37, 2.3, 2.09, 2, 2, 2, 2, 2, 1.98, 1.95, 1.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ] );
		ssc.data_set_number( data, 'wind_turbine_cutin', 4 );
		ssc.data_set_number( data, 'wind_turbine_hub_ht', 15 );
		ssc.data_set_number( data, 'wind_turbine_max_cp', 0.42 );
		ssc.data_set_array( data, 'wind_farm_xCoordinates', [ 0 ] );
		ssc.data_set_array( data, 'wind_farm_yCoordinates', [ 0 ] );
		ssc.data_set_number( data, 'wind_farm_losses_percent', 0 );
		ssc.data_set_number( data, 'wind_farm_wake_model', 0 );
		ssc.data_set_number( data, 'adjust:constant', 0 );	
		


	def run_windmodel( ssc, data ):
		# run PV system simulation
		mod = ssc.module_create("windpower")	
		ssc.module_exec_set_print( 0 );
		if ssc.module_exec(mod, data) == 0:
			print ('wind power simulation error')
			idx = 1
			msg = ssc.module_log(mod, 0)
			while (msg != None):
				print ('\t: ' + msg)
				msg = ssc.module_log(mod, idx)
				idx = idx + 1
		else:
			ann = ssc.data_get_number(data, "annual_energy")
			print ('wind power Simulation ok, annual energy (kwh) =', ann)

		ssc.module_free(mod)

	
	def run_test1():
		wf = './WY Southern-Flat Lands.srw';
		print (wf)
		
		ssc = PySSC()
		dat = ssc.data_create()
		setup_wind(ssc,dat);
		ssc.data_set_string( dat, 'wind_resource_filename', wf );
		run_windmodel( ssc, dat );
		
		
		ssc.data_clear(dat);
		
		# read a weather file for this example program
		# and extract the data from it into a bunch of Python variables
		# note: this weather data could come from any source
			
		# create an SSC data with a bunch of fields
		wfd = ssc.data_create();
		ssc.data_set_number( wfd, 'lat', 0);
		ssc.data_set_number( wfd, 'lon', 0);
		ssc.data_set_number( wfd, 'elev',  2088);
		
		# setup column data types: temp=1,pres=2,3=3,dir=4	
		ssc.data_set_array( wfd, 'fields', [1,2,4,3,1,2,4,3,1,2,4,3,1,2,4,3] );
		
		# setup column measurement heights (meters)
		ssc.data_set_array( wfd, 'heights', [ 50,50,50,50,80,80,80,80,110,110,110,110,140,140,140,140 ] );
		
		# read in the matrix of data corresponding to fields and heights above (should have 8760 rows)	
		data = numpy.loadtxt(open(wf,"rb"),delimiter=",",skiprows=5);
		ssc.data_set_matrix( wfd, 'data', data );
		
		# instead of setting a string weather file, simply
		# set the table variable that contains the various fields
		# with solar resource data
		ssc.data_set_table( dat, 'wind_resource_data', wfd );	
		
		# we can free the resource data table now, since
		# the previous line copies it all into SSC
		ssc.data_free(wfd);
		
		# set up other PV parameters and run
		setup_wind( ssc,dat );
		run_windmodel( ssc, dat );
		
		ssc.data_free(dat)


	def setup_pv(ssc, data):
		ssc.data_set_number( data, 'system_capacity', 4 )
		ssc.data_set_number( data, 'module_type', 0 )
		ssc.data_set_number( data, 'array_type', 0 )
		ssc.data_set_number( data, 'losses', 14 )
		ssc.data_set_number( data, 'tilt', 15 )
		ssc.data_set_number( data, 'azimuth', 180 )
		ssc.data_set_number( data, 'adjust:constant', 0 )

	def run_pvwattsv5( ssc, data ):
		# run PV system simulation
		mod = ssc.module_create("pvwattsv5")	
		ssc.module_exec_set_print( 0 );
		if ssc.module_exec(mod, data) == 0:
			print ('PVWatts V5 simulation error')
			idx = 1
			msg = ssc.module_log(mod, 0)
			while (msg != None):
				print ('\t: ' + msg)
				msg = ssc.module_log(mod, idx)
				idx = idx + 1
		else:
			ann = ssc.data_get_number(data, "ac_annual")
			print ('PVWatts V5 Simulation ok, e_net (annual kW)=', ann)

		ssc.module_free(mod)

	def run_test2():
		#wf = 'c:/Users/adobos/Projects/SAMnt/tests/Weather Files/user-germany-potsdam-2011-1-min-samcsv.csv' ;
		wf = './USA NC Greensboro (TMY2).csv';

		print (wf)
		
		ssc = PySSC()
		dat = ssc.data_create()
		setup_pv(ssc,dat);
		ssc.data_set_string( dat, 'solar_resource_file', wf );
		run_pvwattsv5( ssc, dat );
		
		
		ssc.data_clear(dat);
		
		# read a weather file for this example program
		# and extract the data from it into a bunch of Python variables
		# note: this weather data could come from any source
		ssc.data_set_string( dat, 'file_name', wf );
		ssc.module_exec_simple_no_thread( 'wfreader', dat );	
		lat = ssc.data_get_number(dat, 'lat');
		lon = ssc.data_get_number(dat, 'lon');
		tz = ssc.data_get_number(dat, 'tz');
		elev = ssc.data_get_number(dat, 'elev');
		year = ssc.data_get_array(dat, 'year');
		month = ssc.data_get_array(dat, 'month')
		day = ssc.data_get_array(dat, 'day');
		hour = ssc.data_get_array(dat, 'hour');
		minute = ssc.data_get_array(dat, 'minute');
		beam = ssc.data_get_array(dat, 'beam');
		diffuse = ssc.data_get_array(dat, 'diffuse');
		wspd = ssc.data_get_array(dat, 'wspd');
		tdry = ssc.data_get_array(dat, 'tdry');
		albedo = ssc.data_get_array(dat, 'albedo');	
		ssc.data_clear( dat );
		
		# create an SSC data with a bunch of fields
		wfd = ssc.data_create();
		ssc.data_set_number( wfd, 'lat', lat);
		ssc.data_set_number( wfd, 'lon', lon);
		ssc.data_set_number( wfd, 'tz',  tz);
		ssc.data_set_number( wfd, 'elev',  elev);
		
		ssc.data_set_array( wfd, 'year',  year);
		ssc.data_set_array( wfd, 'month',  month);
		ssc.data_set_array( wfd, 'day',  day);
		ssc.data_set_array( wfd, 'hour', hour);
		
		# note: if using an hourly TMY file with integrated/averaged
		# values, do not set the minute column here. otherwise
		# SSC will assume it is instantaneous data and will not adjust
		# the sun position in sunrise and sunset hours appropriately
		# however, if using subhourly data or instantaneous NSRDB data
		# do explicitly provide the minute data column for sunpos calcs
		
		# ssc.data_set_array( wfd, 'minute', minute);
		
		
		ssc.data_set_array( wfd, 'dn', beam);
		ssc.data_set_array( wfd, 'df', diffuse);
		ssc.data_set_array( wfd, 'wspd', wspd);
		ssc.data_set_array( wfd, 'tdry', tdry);
		ssc.data_set_array( wfd, 'albedo', albedo);
		
		# instead of setting a string weather file, simply
		# set the table variable that contains the various fields
		# with solar resource data
		ssc.data_set_table( dat, 'solar_resource_data', wfd );	
		
		# we can free the resource data table now, since
		# the previous line copies it all into SSC
		ssc.data_free(wfd);
		
		# set up other PV parameters and run
		setup_pv( ssc,dat );
		run_pvwattsv5( ssc, dat );
		
		
		ssc.data_free(dat);


	run_test1();
	run_test2();






import sys
import log
import time
import pandas as pd
from pathlib import Path
from context import *


data_path.mkdir(exist_ok=True, parents=True)
logger = log.custom_logger(__name__)

def timeit(method):
    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()
        if 'log_time' in kw:
            name = kw.get('log_name', method.__name__.upper())
            kw['log_time'][name] = int((te - ts) * 1000)
        else:
            print ('%r  %2.2f ms' % \
                  (method.__name__, (te - ts) * 1000))
        return result
    return timed

def _create_data_folder(path, source='NSRDB', year='2017'):
    """ Create data folder
    Args:
    Returns:
        bool: True
    """
    TIMESERIES_DIR = path / 'raw' / source / 'timeseries' / year
    META_DIR = path / 'raw' / source / 'meta' / year
    TIMESERIES_DIR.mkdir(exist_ok=True, parents=True)
    META_DIR.mkdir(exist_ok=True, parents=True)

    return (TIMESERIES_DIR, META_DIR)

def get_solar_data(file_path=data_path,
                   tmy=False, **kwargs):
    """ Read solar data from data_path

    Read csv file and modify the custom index by a DateTimeIndex using
    columns ['Year', 'Month', 'Day', 'Hour', 'Minute']
    Args:
        path (os.PathLike): data folder direction
        filename (str): data filename inside the data folder
        tmy (str): tmy data type

    Returns
        pd.DataFrame: data with modified index
    """
    #  file_path = data_path / filename + '.csv'

    data = pd.read_csv(file_path)

    datetime_columns = ['Year', 'Month', 'Day', 'Hour', 'Minute']

    if not 'tmy' in file_path.parts:
        try:
            data.index = pd.to_datetime(data[datetime_columns])
        except TypeError:
            data.index = pd.to_datetime(data.index)
        data = data.drop(datetime_columns, axis=1)
    else:
        data.index = pd.to_datetime(data[datetime_columns])
        data = data.drop(datetime_columns, axis=1)
    return (data)

def check_nsrdb(data):
    """ NSRDB FIX
    This function is to fix a bug that prevent to use sam.

    07/25/2017: BUG1 if DHI or any irradiation is negative the code fails.
        Temporal fix: Change negative values to 0
    """
    data = data[data['DHI'] < 0] = 0
    return (data)





# -*- coding: utf-8 -*-
"""
Solar database
Example:
    Example of usage goes here

Attributes:
    module_level_variabless: None for the moment
Todo:
    * Implement multiple inputs and outputs for LCOE.
    * Clean code
"""
import os
import click
import requests
import pandas as pd

import log
from utils import _create_data_folder, get_solar_data
from utils import timeit
from pathlib import Path
from nsrdb import get_nsrdb_data
from context import *


# Module level variables
data_path.mkdir(exist_ok=True, parents=True)
logger = log.custom_logger(__name__)


@click.command()
@click.option("--lat", default=19.5, prompt="Latitude", help="Latitude of the place")
@click.option("--lng", default=-99.5, prompt="Longitude", help="Longitude of the place")
@click.option("--year", default=2014, prompt="Year", help="Year of data up to 2016")
@click.option(
    "--filename",
    default=False,
    prompt="Filename",
    help="Filename of the data",
    required=False,
)
@click.option(
    "--force_download", default=False, is_flag=True, help="Force download from the API"
)
@click.option("--verbose", is_flag=True)
@click.pass_context
def nsrdb(ctx, lat, lng, filename, force_download, year, verbose):
    click.secho("\nGetting data...\n", fg="blue")

    # Define input dictionary
    site_info = {
        "lat": lat,
        "lng": lng,
        "force_download": force_download,
        "year": str(year),
        "filename": filename,
        "verbose": verbose,
    }

    if verbose:
        click.secho("+--Input data", fg="yellow")
        pp.pprint(site_info)
        print("\n")

    data = get_nsrdb_data(**site_info)

    # Print 24 hours
    print(data.head(24))

if __name__ == "__main__":
    nsrdb(obj={})



from __future__ import division, print_function

import pandas as pd
import random
from timezonefinder import TimezoneFinder

def convert_to_local_time(row):
    """ Convert to UTC time
    # Under construction
    Args: dataframe.row
    """
    if row.timezone < 0:
        return pd.to_datetime(row.datetime).tz_localize('Etc/GMT+'+str(abs(row.timezone)))
    else:
        return pd.to_datetime(row.datetime).tz_localize('Etc/GMT-'+str(abs(row.timezone)))
    return pd.to_datetime(row.datetime).tz_localize('Etc/GMT'+str(-row.timezone))

def convert_to_mexico_time(row):
    """ Convert from UTC local time to Mexico Time.
    # Under construction!
    Args: dataframe.row
    """
    return pd.to_datetime(row.datetime_local).tz_convert('Mexico/General')

def convert_to_year(row):
    """ Temporal solution. Probably a future Bug
    # TODO: Fix to a better soluation

    Push data to be in one year only Switch requirements.

    """
    if row.datetime_mexico.year > 2014:
        return (row.datetime_mexico - pd.Timedelta(days=365))
    elif row.datetime_mexico.year < 2014:
        return (row.datetime_mexico + pd.Timedelta(days=365))
    else:
        return (row.datetime_mexico)

def get_solar_data(path, filename, *args, **kwargs):
    """ Read data from folder and process it.
    # Under construction!
    Args:
    path
    filename
    """
    data = pd.read_csv(path + filename + '.csv')
    data = data.drop(data.columns[0], axis=1)
    try:
        data.index = pd.to_datetime(data[['Year', 'Month', 'Day', 'Hour', 'Minute']], utc=True)
    except TypeError:
        data.index = pd.to_datetime(data.index)
    data = data.drop(['Year', 'Month', 'Day', 'Hour', 'Minute'], axis=1)
    data['project_name'] = filename
    data["datetime"] = data.index
    data['datetime_local'] = data.apply(convert_to_local_time, axis=1)
    data['datetime_mexico'] = data.apply(convert_to_mexico_time, axis=1)
    data['datetime_norm'] = data.apply(convert_to_year, axis=1)
    data.index = data['datetime_norm']
    data = data.sort_index()
    data['hour'] = range(1, len(data.index) + 1)
    return (data[['generation', 'hour', 'project_name']])


if __name__ == '__main__':
    directory = '../data/raw/NSRDB/'
    df = pd.read_csv('../data/interim/solar_clusters_centroids.csv')
    random = False
    grouped = df.groupby('lz')
    if random:
        df_sampled = pd.concat([d.loc[random.sample(d.index, 1)] for _, d in grouped]).reset_index(drop=True)
    else:
        #  df.itertuples() is much faster. Will change this if data gets heavy
        data_test_solar = pd.concat([get__solar_data(directory, filename)
            for filename in df['project_name']])
    data_test_solar.to_csv('solar_output.csv')



 # -*- coding: utf-8 -*-
"""
TODO:
    Change log information
"""
import logging

def custom_logger(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)

    # create a file handler
    handler = logging.FileHandler('sam.log')
    handler.setLevel(logging.INFO)

    # create a logging format
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger



"""

"""
import os

import pandas as pd
import requests


PVWATTS_URL = "https://developer.nrel.gov/api/pvwatts/v6.json"


def request_pvwatts(
    lat,
    lon,
    year,
    system_capacity,
    module_type,
    array_type,
    tilt,
    azimuth,
    losses,
    verbose=None,
    force_download=None,
    **kwargs
):
    # TODO: Implement a better way to handle if api_key is not found.
    # Use environment variable if not found in kwargs
    if not "api_key" in kwargs:
        api_key = os.getenv("api_key")  # kwargs['api_key'] # Personal API key
    else:
        api_key = kwargs["api_key"]

    # Raise if api_key not found
    if not api_key:
        raise NameError("api_key not found. Please included it in .env or manually")

    required_parameters = {
        "lat": lat,
        "lon": lon,
        "api_key": api_key,
        "module_type": module_type,
        "system_capacity": system_capacity,  # kw
        "array_type": array_type,
        "tilt": tilt,
        "azimuth": azimuth,
        "losses": losses,
    }

    parameters = {**required_parameters, **kwargs}
    response = requests.get(PVWATTS_URL, params=parameters)

    output_data = response.json()["outputs"]

    keys = output_data.copy().keys()
    for key in keys:
        if key.endswith("_monthly"):
            output_data.pop(key)
    try:
        df = pd.DataFrame(output_data)
    except:
        return output_data
    return df


if __name__ == "__main__":
    site_info = {
        "lat": 33.21,
        "lon": -97.12,
        "module_type": 0,
        "system_capacity": 1000,  # kw
        "array_type": 0,
        "tilt": 20,
        "azimuth": 180,
        "losses": 15,
        "force_download": True,
        "year": 2011,
        "verbose": True,
        # "timeframe": "hourly",
    }
    df = request_pvwatts(**site_info)
    print(df)



import sys
import click
import requests
import pprint

from __version__ import __version__
from context import *
from nsrdb_cli import nsrdb
from nsrdb_cli import get_nsrdb_data
from sam_simulation_cli import sam




CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    version = __version__
    click.echo(f'SWC {version}. ❤ ')
    ctx.exit()

@click.group(invoke_without_command=True)
@click.option('--version', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True)
@click.pass_context
def main(ctx):
    click.echo('Welcome to SWC a wrapper of NSRDB and SAM 🔥!\n')
    click.secho('Starting the CLI\n', fg='blue')
    pass


main.add_command(nsrdb)
main.add_command(sam)

if __name__ == "__main__":
    main(obj={})



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['swc']

package_data = \
{'': ['*']}

install_requires = \
['bump2version>=1.0.0,<2.0.0',
 'click>=7.1.2,<8.0.0',
 'pandas>=1.0.5,<2.0.0',
 'python-dotenv>=0.13.0,<0.14.0',
 'requests>=2.24.0,<3.0.0',
 'tqdm>=4.46.1,<5.0.0']

setup_kwargs = {
    'name': 'swc',
    'version': '0.6.4',
    'description': 'Solar PV performance simulator',
    'long_description': None,
    'author': 'pesap',
    'author_email': 'pesapsanchez@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



""" NSRDB data module

This script downloads the data from the NSRDB from NREL for a specific
location in the world. This toolfunction with an input dictionary with
the parameters to request the data.

This script requires `pandas` and `request` to be insalled within the python
environment you are running this script

TODO:
    * Change way to handle timeseries_path and meta_path
    * Catch different errors from request
    * Implement a better way to debug
"""

import os
import log
import requests
import pandas as pd
from pathlib import Path

# Package scripts
from utils import _create_data_folder, get_solar_data
from utils import timeit

# Package level variables
from context import *

# Creating datafolder
data_path.mkdir(exist_ok=True, parents=True)
logger = log.custom_logger(__name__)

def nsrdb(lat, lng, year, filename=None, force_download=False,
          verbose=False, **kwargs):
    """ NSRDB data

    This function pre-process the request.

    NOTE: This function might change in the future
    Parameters
    ----------
        lat (float):
            Latitude in degrees
        lng (float):
            Longtitude in degrees
        year (str):
            Year of data
        filename (str): optional
            Filename of the data
        force_download (bool): optional
            Force the data to be download. Otherwise look for data in folder.
        verbose (bool): optional
            Print dictionaries
        kwargs (dict):
            Dictionary with api_key to request data

    Returns
    -------
        data (pd.DataFrame):
            Radiation dataDescription of return value
    """
    site_info = {'lat': lat,
                 'lng': lng,
                 'force_download': force_download,
                 'year': str(year),
                 'filename': filename,
                 'verbose': verbose
                }

    # Include kwargs to dictionary
    site_info.update(kwargs)

    if verbose:
        pp.pprint(site_info)

    data = get_nsrdb_data(**site_info)

    return data

def get_nsrdb_data(lat, lng, year, filename=None, path=data_path,
                   force_download=False, timeseries=True,
                   meta=False,  **kwargs):
    """ Get the solar radiation data

    Calls request_nsrdb_data if data is not found in path.

    Parameters
    ----------
        lat (float):
            Latitude in degrees
        lng (float):
            Longtitude in degrees
        year (str):
            Year of data
        filename (str): optional
            Filename of the data
        path (os.PathLike):
            Path to save the data
        force_download (bool): optional
            Force the data to be download. Otherwise look for data in folder.
        timeseries (bool): optional
            false or true for returing timeseries data
        meta (bool): optional
            false or true for returing meta data
        kwargs (dict):
            Dictionary with api_key to request data

    Returns
    -------
        bool: Description of return value
    """

    # If you don't provide a filename use lat and lng.
    if not filename:
        filename = f'{lat:.4f}_{lng:.4f}.csv'
        logger.warning('No filename provided.'
                       'Using Longitude and latitude for filename instead')
    else:
        # Include .csv to filename if it does not have it.
        if filename[-4:] != '.csv':
            filename = filename + '.csv'

    timeseries_path, meta_path = _create_data_folder(path, year=year)
    timeseries_filename = timeseries_path.joinpath(filename)
    meta_path = meta_path.joinpath(filename)

    #  if kwargs['verbose']: print(kwargs)
    #  if kwargs['verbose']:
    #      logger.warning(f'\nSaving data in {timeseries_path}')

    # If force download or file does not exists
    if force_download or not timeseries_filename.is_file():
        logger.info('Downloading timeseries from NSRDB.')
        request_nsrdb_data(lat, lng, year,
                           timeseries_filename=timeseries_filename,
                           meta_path=meta_path, **kwargs)
        logger.info(f'Data created in {timeseries_filename}')
    else:
        logger.info(f'File found in {timeseries_filename}')

    # If you want metadata only
    if meta:
        file_path = meta_path.joinpath(filename)
        meta = pd.read_csv(meta_path)
        return (meta)

    if timeseries:
        #  data = pd.read_csv(time_series_path + filename + '.csv',  index_col=0)
        data = get_solar_data(file_path=timeseries_filename)
        #  data = check_nsrdcb(data)
        return (data)


def request_nsrdb_data(lat, lng, year, timeseries_filename, meta_path,
    utc='false', leap_year='false', interval=60, **kwargs):
    """ NRSDB request function

    Request the radiation data from the NSRDB API. Default columns requested.

    If needed more columns a modification to attributes variables is needed.

    UTC by default is false because if the data is going to be used with SAM
    it needs to be in the local time zone. If used with SWITCH needs to be true
    because switch load projections are in UTC.

    Parameters
    ----------
        lat (float):
            Latitude in degrees
        lng (float):
            Longtitude in degrees
        year (str):
            Year of data
        timeseries_filename (str):
            Filename of the timeseries data
        meta_path (str):
            Folder path of the meta data
        utc (str): optional
            false or true for utc time data
        leap_year (str): optional
            Corresponding to the year above
        interval (str): optional
            This value determines data resolution. Either 30 minute or 60
            minute intervals are available.
        kwargs (dict):
            Dictionary with api_key to request data

    Returns
    ----------
        nothing
    """

    # TODO: Implement a better way to handle if api_key is not found.
    # Use environment variable if not found in kwargs
    if not 'api_key' in kwargs:
        api_key = os.getenv('api_key') #kwargs['api_key'] # Personal API key
    else:
        api_key = kwargs['api_key']

    # Raise if api_key not found
    if not api_key:
        raise NameError('api_key not found. Please included it in .env or manually')

    # TODO: Include more attributes using kwargs.
    if not year == 'tmy':
        attributes = ('ghi,dhi,dni,wind_speed,wind_direction,'
                      'air_temperature,solar_zenith_angle')
    else:
        logger.info('tmy attributes selected')
        attributes = ('ghi,dhi,dni,wind_speed,'
                     'air_temperature')

    # New url to include physical model 3
    url = 'https://developer.nrel.gov/api/solar/nsrdb_psm3_download.csv'

    params = {
        'api_key': api_key,
        'wkt': f'POINT({lng:.4f} {lat:.4f})',
        'attributes': attributes,
        'email': 'mail@gmail.com',
        'names': year, # Year information
        'interval' : str(interval),
        'utc': utc,
        'leap_year': leap_year,
    }

    # Include kwargs in the params
    params.update(kwargs)

    # If verbose print params to request
    if "verbose" in kwargs:
        pp.pprint(params)

    # TODO: Check if the code below is mandatory to make the request.
    # headers = {
        # 'content-type': "application/x-www-form-urlencoded",
        # 'cache-control': "no-cache"
    # }

    # Make the request
    response = requests.get(url, params=params)
    logger.info(f'API Response: {response.status_code}')
    try:
        # Check if limit in response headers
        limit = int(response.headers['X-RateLimit-Remaining'])
        logger.info(f'Request limit: {limit}')
        if limit <= 10:
            logger.warning(f'You almost reach the daily limit. Be careful!')
    except KeyError:
        # TODO: Catch this error
        pass

    # Check if response succeded.
    if response.status_code != 200:
        # Exit program if response different from 200 and print response.
        logger.error('Data downloading failed. Check API.')
        print (response.text)
        print (response.status_code)
        print (response.reason)
        sys.exit(1)
    else:
        # Read data from response with index col to avoid deleting index
        # after exporting
        data = pd.read_csv(response.url, header=None, index_col=[0])

        # Get the metadata from data
        meta = data[:2].reset_index().copy()
        meta = meta.rename(columns=meta.iloc[0]).drop(0)
        meta.to_csv(meta_path, index=False)

        # Get the timeseries from data
        time_series = data[2:].dropna(axis=1).reset_index().copy()
        time_series = time_series.rename(columns=time_series.iloc[0]).drop(0)
        time_series.to_csv(timeseries_filename, index=False)


if __name__ == "__main__":
    site_info = {'lat': 33.21,
                 'lng': -97.12,
                 'force_download': True,
                 'year': str(2012),
                 'verbose': True,
                 'interval': 60
                }

    df = nsrdb(**site_info)
    print (df.head())



#Code to import packages
import sys
import os
from dotenv import load_dotenv
from pathlib import Path  # python3 only
env_path = Path('.env')
load_dotenv(dotenv_path=env_path)

ROOT = os.path.dirname(os.path.realpath(__file__))
sys.path.insert(0, ROOT)

from cli import main
from .__version__ import __version__

if __name__ == "__main__":
    main()



import os
import sys
import pprint
from pathlib import Path

script_path = os.path.dirname(__file__)
parent_path = os.path.dirname(os.path.dirname(__file__))
data_path = Path('data/')
output_path = Path('data/clean/')
output_path.mkdir(parents=True, exist_ok=True)
pp = pprint.PrettyPrinter(indent=4, width=1)

if __name__ == "__main__":
    print (parent_path, data_path)
